const rateLimit = require('express-rate-limit');
const CrawlerError = require('../services/errors/CrawlerError');

/**
 * 创建请求限制中间件
 * @param {Object} options - 限制选项
 * @returns {Function} 中间件函数
 */
const createRateLimiter = (options = {}) => {
  return rateLimit({
    windowMs: options.windowMs || 15 * 60 * 1000, // 15分钟
    max: options.max || 100, // 限制每个IP 15分钟内最多100个请求
    message: {
      error: '请求过于频繁，请稍后再试',
      code: 'RATE_LIMIT_EXCEEDED'
    }
  });
};

/**
 * 错误处理中间件
 * @param {Error} err - 错误对象
 * @param {Request} req - 请求对象
 * @param {Response} res - 响应对象
 * @param {Function} next - 下一个中间件
 */
const errorHandler = (err, req, res, next) => {
  console.error('Error:', err);

  if (err instanceof CrawlerError) {
    return res.status(400).json({
      success: false,
      error: err.message,
      code: err.code,
      platform: err.platform,
      timestamp: err.timestamp
    });
  }

  // 处理其他类型的错误
  res.status(500).json({
    error: '服务器内部错误',
    code: 'INTERNAL_SERVER_ERROR',
    message: process.env.NODE_ENV === 'development' ? err.message : undefined
  });
};

/**
 * 请求验证中间件
 * @param {Request} req - 请求对象
 * @param {Response} res - 响应对象
 * @param {Function} next - 下一个中间件
 */
const validateRequest = (req, res, next) => {
  const { url, platform } = req.query;

  if (!url) {
    return res.status(400).json({
      error: '缺少必要参数: url',
      code: 'MISSING_PARAMETER'
    });
  }

  if (!platform) {
    return res.status(400).json({
      error: '缺少必要参数: platform',
      code: 'MISSING_PARAMETER'
    });
  }

  try {
    new URL(url);
  } catch (err) {
    return res.status(400).json({
      error: '无效的URL格式',
      code: 'INVALID_URL'
    });
  }

  next();
};

module.exports = {
  createRateLimiter,
  errorHandler,
  validateRequest
}; 