// 加载环境变量
require('dotenv').config();

const express = require('express');
const path = require('path');
const cors = require('cors');
const helmet = require('helmet');
const morgan = require('morgan');
const crawlerRoutes = require('./routes/crawler');
const { errorHandler } = require('./middlewares/crawlerMiddleware');
const envConfig = require('./config/env.config');

// 处理 punycode 弃用警告
process.removeAllListeners('warning');
process.on('warning', (warning) => {
  if (warning.name === 'DeprecationWarning' && warning.message.includes('punycode')) {
    return;
  }
  console.warn(warning);
});

const app = express();

// 安全中间件
app.use(helmet({
  crossOriginResourcePolicy: { policy: "cross-origin" }
}));

// CORS配置
app.use(cors(envConfig.cors));

// 日志中间件
app.use(morgan(envConfig.logging.format));

// 解析 JSON 请求体
app.use(express.json());
// 解析 URL 编码的请求体
app.use(express.urlencoded({ extended: true }));

// 设置静态文件目录
const screenshotsDir = path.join(process.cwd(), 'screenshots');
app.use('/screenshots', express.static(screenshotsDir));

// 获取本机IP地址
const os = require('os');
function getLocalIP() {
  const interfaces = os.networkInterfaces();
  for (const name of Object.keys(interfaces)) {
    for (const iface of interfaces[name]) {
      // 跳过内部IP和非IPv4地址
      if (iface.family === 'IPv4' && !iface.internal) {
        return iface.address;
      }
    }
  }
  return '0.0.0.0'; // 默认监听所有接口
}

// 添加请求日志中间件
app.use((req, res, next) => {
  console.log(`${new Date().toISOString()} - ${req.method} ${req.url}`);
  next();
});

// 路由
app.use('/api', crawlerRoutes);

// 健康检查端点
app.get('/health', (req, res) => {
  res.json({
    status: 'ok',
    timestamp: new Date().toISOString(),
    environment: process.env.NODE_ENV || 'development'
  });
});

// 404 处理
app.use((req, res) => {
  res.status(404).json({
    success: false,
    message: '接口不存在',
    path: req.path
  });
});

// 错误处理中间件
app.use(errorHandler);

// 启动服务器
const server = app.listen(envConfig.port, envConfig.host, () => {
  const localIP = getLocalIP();
  console.log('服务器配置:');
  console.log(`- 环境: ${process.env.NODE_ENV || 'development'}`);
  console.log(`- 监听地址: ${envConfig.host}:${envConfig.port}`);
  console.log(`- 本地访问: http://localhost:${envConfig.port}`);
  console.log(`- 局域网访问: http://${localIP}:${envConfig.port}`);
  console.log(`- 健康检查: http://${localIP}:${envConfig.port}/health`);
});

// 优雅关闭
process.on('SIGTERM', () => {
  console.log('收到 SIGTERM 信号，准备关闭服务器');
  server.close(() => {
    console.log('服务器已关闭');
    process.exit(0);
  });
});

process.on('SIGINT', () => {
  console.log('收到 SIGINT 信号，准备关闭服务器');
  server.close(() => {
    console.log('服务器已关闭');
    process.exit(0);
  });
}); 