const axios = require('axios');

/**
 * 商品爬虫API SDK
 * 提供爬虫服务的完整客户端接口
 */
class CrawlerApiSDK {
  /**
   * 构造函数
   * @param {Object} options - 配置选项
   * @param {string} options.baseURL - API基础URL
   * @param {number} options.timeout - 请求超时时间(毫秒)
   * @param {Object} options.headers - 自定义请求头
   * @param {boolean} options.debug - 是否开启调试模式
   */
  constructor(options = {}) {
    this.options = {
      baseURL: options.baseURL || 'http://localhost:8991',
      timeout: options.timeout || 60000,
      headers: options.headers || {},
      debug: options.debug || false
    };

    // 创建axios实例
    this.httpClient = axios.create({
      baseURL: this.options.baseURL,
      timeout: this.options.timeout,
      headers: {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        ...this.options.headers
      }
    });

    // 请求拦截器
    this.httpClient.interceptors.request.use(
      config => {
        if (this.options.debug) {
          console.log('API请求:', {
            method: config.method.toUpperCase(),
            url: config.url,
            params: config.params,
            headers: config.headers
          });
        }
        return config;
      },
      error => {
        if (this.options.debug) {
          console.error('API请求错误:', error);
        }
        return Promise.reject(error);
      }
    );

    // 响应拦截器
    this.httpClient.interceptors.response.use(
      response => {
        if (this.options.debug) {
          console.log('API响应:', {
            status: response.status,
            statusText: response.statusText,
            url: response.config.url,
            data: response.data
          });
        }
        return response.data;
      },
      error => {
        return this._handleRequestError(error);
      }
    );
  }

  /**
   * 处理请求错误
   * @private
   * @param {Error} error - 错误对象
   * @returns {Promise<Error>} 处理后的错误
   */
  _handleRequestError(error) {
    if (this.options.debug) {
      console.error('API响应错误:', error);
    }

    let errorMessage = '请求失败';
    let errorCode = 'UNKNOWN_ERROR';
    let statusCode = 500;
    let originalError = null;

    if (error.response) {
      // 服务器返回了错误状态码
      statusCode = error.response.status;
      const errorData = error.response.data;

      if (errorData) {
        if (errorData.error) {
          errorMessage = typeof errorData.error === 'string'
            ? errorData.error
            : errorData.error.message || errorMessage;
        }
        errorCode = errorData.code || `HTTP_${statusCode}`;
      }
    } else if (error.request) {
      // 请求已发送但没有收到响应
      errorMessage = '服务器无响应，请检查网络连接';
      errorCode = 'CONNECTION_ERROR';
      statusCode = 0;
    } else {
      // 设置请求时发生错误
      errorMessage = error.message || errorMessage;
      errorCode = 'REQUEST_SETUP_ERROR';
    }

    originalError = error;

    // 创建增强的错误对象
    const enhancedError = new Error(errorMessage);
    enhancedError.statusCode = statusCode;
    enhancedError.code = errorCode;
    enhancedError.originalError = originalError;

    return Promise.reject(enhancedError);
  }

  /**
   * 获取商品信息
   * @param {Object} params - 请求参数
   * @param {string} params.url - 商品URL
   * @param {string} params.platform - 平台名称，如'amazon'
   * @param {boolean} [params.needScreenshot=false] - 是否需要截图
   * @returns {Promise<Object>} 商品信息
   * @throws {Error} 如果请求失败
   */
  async getProductInfo(params) {
    // 参数验证
    if (!params || typeof params !== 'object') {
      throw new Error('参数必须是一个对象');
    }

    if (!params.url) {
      throw new Error('商品URL不能为空');
    }

    if (!params.platform) {
      throw new Error('平台名称不能为空');
    }

    // 构建查询参数
    const queryParams = {
      url: params.url,
      platform: params.platform.toLowerCase(),
      needScreenshot: params.needScreenshot === true ? 'true' : 'false'
    };

    try {
      const response = await this.httpClient.get('/api/product/info', {
        params: queryParams
      });
      
      if (!response.success) {
        throw new Error(response.error || '获取商品信息失败');
      }
      
      return response.data;
    } catch (error) {
      throw error;
    }
  }

  /**
   * 获取支持的平台列表
   * @returns {Promise<string[]>} 支持的平台列表
   * @throws {Error} 如果请求失败
   */
  async getSupportedPlatforms() {
    try {
      const response = await this.httpClient.get('/api/platforms');
      
      if (!response.success) {
        throw new Error(response.error || '获取平台列表失败');
      }
      
      return response.data;
    } catch (error) {
      throw error;
    }
  }

  /**
   * 健康检查
   * @returns {Promise<Object>} 健康状态信息
   * @throws {Error} 如果请求失败
   */
  async healthCheck() {
    try {
      return await this.httpClient.get('/health');
    } catch (error) {
      throw error;
    }
  }
}

module.exports = CrawlerApiSDK; 