const BaseCrawler = require("../base/BaseCrawler");
const CrawlerError = require("../../errors/CrawlerError");
const path = require("path");
const fs = require("fs");
const fsPromises = require("fs").promises;
const axios = require("axios");
const FormData = require("form-data");

/**
 * 乐天爬虫实现类
 */
class RakutenCrawler extends BaseCrawler {
  constructor(config) {
    super(config);
    console.log(config);
    this.selectors = config.selectors;
    this.timeouts = config.timeouts;
    this.retryConfig = config.retry;
    this.uploadConfig = config.common?.upload;
  }

  /**
   * 创建截图目录
   * @returns {Promise<string>} 截图目录路径
   */
  async createScreenshotDir() {
    const dir = path.join(process.cwd(), "screenshots");
    try {
      await fsPromises.mkdir(dir, { recursive: true });
    } catch (error) {
      if (error.code !== "EEXIST") {
        throw new CrawlerError(
          "创建截图目录失败",
          "SCREENSHOT_DIR_ERROR",
          "rakuten",
          error
        );
      }
    }
    return dir;
  }

  /**
   * 上传图片到服务器
   * @param {string} imagePath - 图片路径
   * @returns {Promise<string>} 图片URL
   */
  async uploadImage(imagePath) {
    try {
      const formData = new FormData();
      formData.append("file", fs.createReadStream(imagePath));
      formData.append("scene", this.uploadConfig.scene);

      const response = await axios.post(this.uploadConfig.url, formData, {
        headers: {
          ...formData.getHeaders(),
          "Content-Type": "multipart/form-data",
        },
        timeout: this.uploadConfig.timeout,
      });

      if (!response.data || !response.data.url) {
        throw new Error("上传响应格式错误");
      }

      return response.data.url;
    } catch (error) {
      if (error.response) {
        throw new CrawlerError(
          `图片上传失败: ${error.response.status} ${error.response.statusText}`,
          "IMAGE_UPLOAD_ERROR",
          "rakuten",
          error
        );
      }
      throw new CrawlerError(
        "图片上传失败",
        "IMAGE_UPLOAD_ERROR",
        "rakuten",
        error
      );
    }
  }

  /**
   * 带重试的页面导航
   * @param {string} url - 目标URL
   * @returns {Promise<void>}
   */
  async navigateWithRetry(url) {
    let lastError;
    for (let attempt = 1; attempt <= this.retryConfig.maxAttempts; attempt++) {
      try {
        await this.page.goto(url, {
          waitUntil: "networkidle",
          timeout: this.timeouts.pageLoad,
        });
        return;
      } catch (error) {
        lastError = error;
        console.log(
          `导航尝试 ${attempt}/${this.retryConfig.maxAttempts} 失败:`,
          error.message
        );

        if (attempt < this.retryConfig.maxAttempts) {
          console.log(`等待 ${this.retryConfig.delay}ms 后重试...`);
          await new Promise((resolve) =>
            setTimeout(resolve, this.retryConfig.delay)
          );

          // 重新初始化浏览器
          await this.closeBrowser();
          await this.initBrowser();
        }
      }
    }
    throw new CrawlerError(
      `页面导航失败，已重试 ${this.retryConfig.maxAttempts} 次`,
      "NAVIGATION_ERROR",
      "rakuten",
      lastError
    );
  }

  /**
   * 等待元素出现
   * @param {string} selector - 选择器
   * @returns {Promise<void>}
   */
  async waitForElement(selector) {
    try {
      await this.page.waitForSelector(selector, {
        timeout: this.timeouts.elementWait,
      });
    } catch (error) {
      throw new CrawlerError(
        `等待元素超时: ${selector}`,
        "ELEMENT_WAIT_ERROR",
        "rakuten",
        error
      );
    }
  }

  /**
   * 获取单个SKU信息
   * @returns {Promise<Object>} SKU信息
   */
  async getSingleSkuInfo() {
    try {
      // 等待页面加载完成
      await this.page.waitForLoadState("networkidle");

      // 等待标题元素出现
      await this.waitForElement(this.selectors.title);

      // 处理优惠券
      // const couponValue = await this.handleCoupon();

      // 获取商品信息
      const info = await this.page.evaluate(
        ({ selectors, couponValue }) => {
          const title =
            document.querySelector(selectors.title)?.textContent.trim() || null;
          let priceText =
            document.querySelector(selectors.price)?.textContent.trim() || null;

          // 解析价格
          const price = priceText;
          const url = window.location.href;
          const asin =
            document
              .querySelector(".normal_reserve_item_number")
              ?.textContent.trim() || null;

          return {
            title,
            price: price ? price.toLocaleString() : null,
            sku: asin,
            url,
          };
        },
        { selectors: this.selectors, couponValue }
      );


      console.log(info);

      // 验证必要信息
      if (!info.title || !info.price || !info.sku) {
        throw new Error("商品信息不完整");
      }

      return info;
    } catch (error) {
      throw new CrawlerError(
        "获取SKU信息失败",
        "SKU_INFO_GET_ERROR",
        "rakuten",
        error
      );
    }
  }

  /**
   * 主方法：抓取商品信息
   * @param {string} url - 商品URL
   * @param {boolean} needScreenshot - 是否需要截图
   * @returns {Promise<Array>} 商品信息数组
   */
  async crawl(url, needScreenshot = false) {
    try {
      await this.initBrowser();

      // 设置页面超时
      this.page.setDefaultTimeout(this.timeouts.elementWait);
      this.page.setDefaultNavigationTimeout(this.timeouts.pageLoad);

      // 设置请求拦截
      await this.page.route("**/*", (route) => {
        const resourceType = route.request().resourceType();
        // 只阻止字体和媒体资源，允许加载图片
        if (["font", "media"].includes(resourceType)) {
          route.abort();
        } else {
          route.continue();
        }
      });

      // 导航到目标页面
      await this.navigateWithRetry(url);

      // 只获取单个SKU信息
      const data = [await this.getSingleSkuInfo()];

      if (needScreenshot) {
        try {
          const dir = await this.createScreenshotDir();
          const filename = `${Date.now()}.png`;
          const shot = path.join(dir, filename);

          // 等待页面完全加载
          await this.page.waitForLoadState("networkidle");

          // 截取全页面
          await this.page.screenshot({
            path: shot,
            fullPage: true,
            timeout: this.timeouts.elementWait,
          });

          // 上传图片并获取URL
          const imageUrl = await this.uploadImage(shot);

          // 更新数据，添加图片URL
          data.forEach((item) => {
            item.screenshotUrl = imageUrl;
          });

          // 删除临时文件
          try {
            await fsPromises.unlink(shot);
          } catch (error) {
            console.error("删除临时截图文件失败:", error);
          }
        } catch (error) {
          console.error("截图处理失败:", error);
          // 截图失败不影响主流程
        }
      }

      return data;
    } catch (error) {
      throw new CrawlerError(
        "商品信息抓取失败",
        "CRAWL_ERROR",
        "rakuten",
        error
      );
    } finally {
      await this.closeBrowser();
    }
  }
}

module.exports = RakutenCrawler;
