const axios = require('axios');

/**
 * 商品API客户端服务
 * 用于调用爬虫API获取商品信息
 */
class ProductApiClient {
  /**
   * 构造函数
   * @param {Object} options - 配置选项
   * @param {string} options.baseURL - API基础URL
   * @param {number} options.timeout - 请求超时时间(毫秒)
   * @param {Object} options.headers - 请求头
   */
  constructor(options = {}) {
    this.baseURL = options.baseURL || 'http://localhost:8991';
    this.timeout = options.timeout || 60000;
    this.headers = options.headers || {
      'Content-Type': 'application/json',
      'Accept': 'application/json'
    };
    
    // 创建axios实例
    this.client = axios.create({
      baseURL: this.baseURL,
      timeout: this.timeout,
      headers: this.headers
    });
    
    // 添加响应拦截器
    this.client.interceptors.response.use(
      response => response.data,
      error => this.handleError(error)
    );
  }
  
  /**
   * 处理请求错误
   * @private
   * @param {Error} error - 错误对象
   * @throws {Error} 处理后的错误
   */
  handleError(error) {
    let errorMessage = '获取商品信息失败';
    let statusCode = 500;
    
    if (error.response) {
      // 服务器返回了错误状态码
      statusCode = error.response.status;
      const errorData = error.response.data;
      
      if (errorData && errorData.error) {
        errorMessage = typeof errorData.error === 'string' 
          ? errorData.error 
          : errorData.error.message || errorMessage;
      }
    } else if (error.request) {
      // 请求已发送但没有收到响应
      errorMessage = '服务器无响应，请检查网络连接';
      statusCode = 0;
    }
    
    const enhancedError = new Error(errorMessage);
    enhancedError.statusCode = statusCode;
    enhancedError.originalError = error;
    
    return Promise.reject(enhancedError);
  }
  
  /**
   * 获取商品信息
   * @param {Object} params - 请求参数
   * @param {string} params.url - 商品URL
   * @param {string} params.platform - 平台名称(amazon)
   * @param {boolean} [params.needScreenshot=false] - 是否需要截图
   * @returns {Promise<Object>} 商品信息
   */
  async getProductInfo(params) {
    if (!params.url) {
      throw new Error('商品URL不能为空');
    }
    
    if (!params.platform) {
      throw new Error('平台名称不能为空');
    }
    
    // 构建查询参数
    const queryParams = {
      url: params.url,
      platform: params.platform,
      needScreenshot: params.needScreenshot === true ? 'true' : 'false'
    };
    
    try {
      const response = await this.client.get('/api/product/info', { params: queryParams });
      return response.data;
    } catch (error) {
      console.error('获取商品信息失败:', error);
      throw error;
    }
  }
  
  /**
   * 获取支持的平台列表
   * @returns {Promise<string[]>} 平台列表
   */
  async getSupportedPlatforms() {
    try {
      const response = await this.client.get('/api/platforms');
      return response.data;
    } catch (error) {
      console.error('获取平台列表失败:', error);
      throw error;
    }
  }
}

module.exports = ProductApiClient; 